using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.Schedule;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace GridScheduleSample
{
    #region MDB_ScheduleItem class

    /// <summary>
    /// This class provides a schedule appointment as a record in a DataTable from an MDB file. It wraps
    /// DataRow objects using an IScheduleAppointment interface through the ScheduleAppointment base class.
    /// </summary>
    public class MDB_ScheduleItem : ScheduleAppointment
    {
        /// <summary>
        /// Holds the name of the AllDay column.
        /// </summary>
        public static string Column_AllDay = "AllDay";
        /// <summary>
        /// Holds the name of the Content column.
        /// </summary>
        public static string Column_Content = "Content";
        /// <summary>
        /// Holds the name of the Dirty column.
        /// </summary>
        public static string Column_Dirty = "Dirty";
        /// <summary>
        /// Holds the name of the EndTime column.
        /// </summary>
        public static string Column_EndTime = "EndTime";
        /// <summary>
        /// Holds the name of the IgnoreChanges column.
        /// </summary>
        public static string Column_IgnoreChanges = "IgnoreChanges";
        /// <summary>
        /// Holds the name of the LabelValue column.
        /// </summary>
        public static string Column_LabelValue = "LabelValue";
        /// <summary>
        /// Holds the name of the LocationValue column.
        /// </summary>
        public static string Column_LocationValue = "LocationValue";
        /// <summary>
        /// Holds the name of the MarkerValue column.
        /// </summary>
        public static string Column_MarkerValue = "MarkerValue";
        /// <summary>
        /// Holds the name of the Owner column.
        /// </summary>
        public static string Column_Owner = "Owner";
        /// <summary>
        /// Holds the name of the Reminder column.
        /// </summary>
        public static string Column_Reminder = "Reminder";
        /// <summary>
        /// Holds the name of the ReminderValue column.
        /// </summary>
        public static string Column_ReminderValue = "ReminderValue";
        /// <summary>
        /// Holds the name of the StartTime column.
        /// </summary>
        public static string Column_StartTime = "StartTime";
        /// <summary>
        /// Holds the name of the Subject column.
        /// </summary>
        public static string Column_Subject = "Subject";
        /// <summary>
        /// Holds the name of the ID column.
        /// </summary>
        public static string Column_UniqueID = "ID";
       
       

       /// <summary>
       /// Constructor that accepts a DataRow holding the appointment information.
       /// </summary>
       /// <param name="dr">The DataRow.</param>
        public MDB_ScheduleItem(DataRow dr)
        {
            this.dr = dr;
        }

        private DataRow dr;
        /// <summary>
        /// Gets the DataRow associated with this appointment item.
        /// </summary>
        public DataRow Dr
        {
            get { return dr; }
        }

        /// <summary>
        /// Gets or sets the AllDay value for this appointment.
        /// </summary>
        public override bool AllDay
        {
            get
            {
                return (bool) dr[Column_AllDay];
            }
            set
            {
                dr[Column_AllDay] = value;
            }
        }
        /// <summary>
        /// Gets or sets the Content for this appointment.
        /// </summary>
        public override string Content
        {
            get
            {
                return (string) dr[Column_Content];
            }
            set
            {
                dr[Column_Content] = value;
            }
        }
        /// <summary>
        /// Gets whether the appointment has been modified by checking the DataRow.RecordState.
        /// </summary>
        public override bool Dirty
        {
            get
            {
                return dr.RowState != DataRowState.Unchanged;
            }
            set
            {
                //  nop
            }
        }
        /// <summary>
        /// Gets or sets the EndTime for this appointment.
        /// </summary>
        public override DateTime EndTime
        {
            get
            {
                return (DateTime) dr[Column_EndTime];
            }
            set
            {
                dr[Column_EndTime] = value;
            }
        }
        /// <summary>
        /// Gets of sets the IgnoreChanges property for this appointment.
        /// </summary>
        public override bool IgnoreChanges
        {
            get
            {
                return (bool)dr[Column_IgnoreChanges];
            }
            set
            {
                dr[Column_IgnoreChanges] = value;
            }
        }
        /// <summary>
        /// Gets or sets the LabelValue for this appointment.
        /// </summary>
        public override int LabelValue
        {
            get
            {
                return (int)dr[Column_LabelValue];
            }
            set
            {
                dr[Column_LabelValue] = value;
            }
        }
        /// <summary>
        /// Gets or sets the LocationValue for this appointment.
        /// </summary>
        public override string LocationValue
        {
            get
            {
               return FindLookupValue(MDB_ScheduleDataProvider.Locations, (int) dr[Column_LocationValue]);
            }
            set
            {
                dr[Column_LocationValue] = value;
            }
        }
        //Utility method to map lookup value to display value for the general objects lists.
        private string FindLookupValue(LookUpObjectList list, int itemToFind)
        {
            string val = "";
            foreach (ListObject drv in list)
            {
                if (itemToFind.Equals(drv.ValueMember))
                {
                    val = drv.DisplayMember;
                    break;
                }
            }
            return val;
        }
        /// <summary>
        /// Gets or sets the MarkerValue for this appointment.
        /// </summary>
        public override int MarkerValue
        {
            get
            {
                return (int)dr[Column_MarkerValue];
            }
            set
            {
                dr[Column_MarkerValue] = value;
            }
        }
        /// <summary>
        /// Gets or sets the Owner value for this appointment.
        /// </summary>
        public override int Owner
        {
            get
            {
                return (int)dr[Column_Owner];
            }
            set
            {
                dr[Column_Owner] = value;
            }
        }
        /// <summary>
        /// Gets or sets whether this appointment has a reminder active.
        /// </summary>
        public override bool Reminder
        {
            get
            {
                return (bool)dr[Column_Reminder];
            }
            set
            {
                dr[Column_Reminder] = value;
            }
        }

        /// <summary>
        /// Gets or sets the ReminderValue for this appointment.
        /// </summary>
        public override int ReminderValue
        {
            get
            {
                return (int)dr[Column_ReminderValue];
            }
            set
            {
                dr[Column_ReminderValue] = value;
            }
        }

        /// <summary>
        /// Gets or sets the StartTime for this appointment.
        /// </summary>
        public override DateTime StartTime
        {
            get
            {
                return (DateTime)dr[Column_StartTime];
            }
            set
            {
                dr[Column_StartTime] = value;
            }
        }
        /// <summary>
        /// Gets or sets the Subject value for this appointment.
        /// </summary>
        public override string Subject
        {
            get
            {
                if (dr[Column_Subject] == DBNull.Value)
                    return "";
                return (string)dr[Column_Subject];
            }
            set
            {
                dr[Column_Subject] = value;
            }
        }
        /// <summary>
        /// Gets or sets the UniqueID aassociated with this appointment.
        /// </summary>
        public override int UniqueID
        {
            get
            {
                return (int)dr[Column_UniqueID];
            }
            set
            {
                dr[Column_UniqueID] = value;
            }
        }
    }

    #endregion

    #region MDB_ScheduleItemList class

    /// <summary>
    /// Class that wraps a collection of MDB_ScheduleItem's in a list.
    /// </summary>
    public class MDB_ScheduleItemList : ArrayListAppointmentList
    {
        /// <summary>
        /// Constructor that accepts a DataView and creates a MDB_ScheduleItemList of MDB_ScheduleItem objects.
        /// </summary>
        /// <param name="dv">The DataView holding the appointments.</param>
        public MDB_ScheduleItemList(DataView dv)
        {
            SetDataView(dv);
        }
        /// <summary>
        /// Populates this MDB_ScheduleItemList object with the appointments in the given DataView.
        /// </summary>
        /// <param name="dv"></param>
        protected void SetDataView(DataView dv)
        {
            foreach (DataRowView drv in dv)
            {
                this.Add(new MDB_ScheduleItem(drv.Row));
            }
        }
    }

    #endregion

    #region MDB_ScheduleDataProvider class

    /// <summary>
    /// A ScheduleDataProvider derived class that provides schedule data using
    /// an ADO Dataset and DataTables through an MDB file.
    /// </summary>
    public class MDB_ScheduleDataProvider : ScheduleDataProvider
    {
        /// <summary>
        /// The name of the ADO DataSet.
        /// </summary>
        public static string DataSet_Name = "Schedule";
        /// <summary>
        /// The name of the DataTable holding the appointment information.
        /// </summary>
        public static string Table_Appointments = "Appointments";
        /// <summary>
        /// The name of the DataTable holding the Markers lookup information.
        /// </summary>
        public static string Table_Markers = "Markers";
        /// <summary>
        /// The name of the DataTable holding the Labels loopkup information.
        /// </summary>
        public static string Table_Labels = "Labels";
        /// <summary>
        /// The name of the DataTable holding the Reminders lookup information.
        /// </summary>
        public static string Table_Reminders = "Reminders";
        /// <summary>
        /// The name of teh DataTable holding the Locations loopup information.
        /// </summary>
        public static string Table_Locations = "Locations";
      
        internal static DataSet scheduleDataSet = null;

        /// <summary>
        /// Gets the underlying DataSet associated with this MDB_ScheduleDataProvider.
        /// </summary>
        public DataSet ScheduleDataSet
        {
            get { return scheduleDataSet; }
        }

        private bool loadOnDemand = true;

        /// <summary>
        /// Gets or sets whether the appointments are loaded on demand or not.
        /// </summary>
        /// <remarks>
        /// The default value is true. When this value is true, a live connection is maintained
        /// to the database and only those appointments are currently displayed are loaded into memory.
        /// As you change the display date, any modifications of the current data is saved to the
        /// database, and the new required data is loaded. 
        /// 
        /// When this value is false, the entire appointment table is loaded into memory, and the
        /// program works in a disconnected manner. As you change the display date, no changes are 
        /// pushed back to the database, and the new appointment data is loaded from the memory
        /// cache. When you exit the program, the changes are pushed to the database.
        /// </remarks>
        public bool LoadOnDemand
        {
            get { return loadOnDemand; }
            set { loadOnDemand = value; }
        }


        OleDbDataAdapter dataAdapter;
        OleDbConnection connection;
        private string MDBpathName;

        /// <summary>
        /// Gets the OleDb connection to the MDB file.
        /// </summary>
        /// <remarks>If the coonection is null, it will be created using the pathname provided in the
        /// MDB_ScheduleDataProvider constructor. If the connection is not opened, it will be re-opened.
        /// </remarks>
        public OleDbConnection Connection
        {
            get 
            {
                if (connection == null)
                {
                    string connString = @"Provider=Microsoft.JET.OLEDB.4.0;data source=" + this.MDBpathName;

                    connection = new OleDbConnection(connString);
                }
                if (connection.State == ConnectionState.Closed)
                    connection.Open();

                return connection; 
            }
        }
        private string EmptyMDBFileName = "xScheduleDataEmptyFile";

        /// <summary>
        /// Creates a schedule data provider based on an MDB file.
        /// </summary>
        /// <param name="MDBpathName">The full pathname to the MDB file.</param>
        /// <param name="loadOnDemand">Indicates whether the complete appointment table is loaded, 
        /// or if the appointments are only loaded as needed.
        /// </param>
        /// <remarks>
        /// 
        /// When loadOnDemand is true, a live connection is maintained
        /// to the database and only those appointments are currently displayed are loaded into memory.
        /// As you change the display dates, any modifications of the current loaded data is saved to the
        /// database, and the new required data is loaded. 
        /// 
        /// When loadOnDemand is false, the entire appointment table is loaded into memory, and the
        /// program works in a disconnected manner. As you change the display date, no changes are 
        /// pushed back to the database, and the new appointment data is loaded from the memory
        /// cache. When you exit the program, the changes are pushed to the database.
        /// 
        /// If the MDB file identified by the string held in the MDBpathName variable does not
        /// exist, for the purposes of this sample, a new MDB file is copied to the specified pathname holding no appointments
        /// but with other values such as Locations and Markers. This new file is created by copying
        /// the file named xScheduleDataEmptyFile that is found in the folder with the executable.
        /// </remarks>
        public MDB_ScheduleDataProvider(string MDBpathName, bool loadOnDemand)
        {
            if (!File.Exists(MDBpathName) && File.Exists(EmptyMDBFileName))
            {
                try
                {
                    File.Copy(EmptyMDBFileName, MDBpathName);
                }
                catch
                {
                    MessageBox.Show("Unable to create empty MDB File.");
                    return;
                }
            }
            this.loadOnDemand = loadOnDemand;
           MDB_ScheduleDataProvider.scheduleDataSet = new DataSet(DataSet_Name);

           // string connString = @"Provider=Microsoft.JET.OLEDB.4.0;data source=" + MDBpathName;
         	
           // connection = new OleDbConnection(connString);

            this.MDBpathName = MDBpathName;

            string sqlString = "SELECT * FROM " + Table_Markers;
            dataAdapter = new OleDbDataAdapter(sqlString, Connection);
            dataAdapter.Fill(scheduleDataSet, Table_Markers);

            sqlString = "SELECT * FROM " + Table_Labels;
            dataAdapter = new OleDbDataAdapter(sqlString, Connection);
            dataAdapter.Fill(scheduleDataSet, Table_Labels);

            sqlString = "SELECT * FROM " + Table_Reminders;
            dataAdapter = new OleDbDataAdapter(sqlString, Connection);
            dataAdapter.Fill(scheduleDataSet, Table_Reminders);

            sqlString = "SELECT * FROM " + Table_Locations;
            dataAdapter = new OleDbDataAdapter(sqlString, Connection);
            dataAdapter.Fill(scheduleDataSet, Table_Locations);

            if (!LoadOnDemand)
            {
                sqlString = "SELECT * FROM " + Table_Appointments;
                dataAdapter = new OleDbDataAdapter(sqlString, Connection);
                dataAdapter.Fill(scheduleDataSet, Table_Appointments);
                Connection.Close();
            }
        }
        /// <summary>
        /// Adds an appointment to the Appointments DataTable.
        /// </summary>
        /// <param name="item">The appointment to be added.</param>
        public override void AddItem(IScheduleAppointment item)
        {
            scheduleDataSet.Tables[Table_Appointments].Rows.Add(((MDB_ScheduleItem) item).Dr);
        }

        /// <summary>
        /// Does an Update/Insert/Delete command on the Appointments DataTable.
        /// </summary>
        public override void CommitChanges()
        {
            #region Update Command

            this.dataAdapter.UpdateCommand = new OleDbCommand();
            this.dataAdapter.UpdateCommand.Connection = this.Connection;
            this.dataAdapter.UpdateCommand.CommandText = string.Format(@"UPDATE `{0}` SET `{1}` = ?, `{2}` = ?, `{3}` = ?, `{4}` = ?, `{5}` = ?, `{6}` = ?, `{7}` = ?, `{8}` = ?, `{9}` = ?, `{10}` = ?, `{11}` = ? WHERE ((`{12}` = ?) AND ((? = 1 AND `{1}` IS NULL) OR (`{1}` = ?)) AND ((? = 1 AND `{3}` IS NULL) OR (`{3}` = ?)) AND ((? = 1 AND `{4}` IS NULL) OR (`{4}` = ?)) AND ((? = 1 AND `{5}` IS NULL) OR (`{5}` = ?)) AND ((? = 1 AND `{6}` IS NULL) OR (`{6}` = ?)) AND ((? = 1 AND `{7}` IS NULL) OR (`{7}` = ?)) AND ((? = 1 AND `{8}` IS NULL) OR (`{8}` = ?)) AND ((? = 1 AND `{9}` IS NULL) OR (`{9}` = ?)) AND ((? = 1 AND `{10}` IS NULL) OR (`{10}` = ?)))",
                                                                    Table_Appointments, MDB_ScheduleItem.Column_AllDay, MDB_ScheduleItem.Column_Content, MDB_ScheduleItem.Column_EndTime,
                                                                    MDB_ScheduleItem.Column_LabelValue, MDB_ScheduleItem.Column_LocationValue, MDB_ScheduleItem.Column_MarkerValue,
                                                                    MDB_ScheduleItem.Column_Owner, MDB_ScheduleItem.Column_Reminder, MDB_ScheduleItem.Column_ReminderValue, MDB_ScheduleItem.Column_StartTime, MDB_ScheduleItem.Column_Subject, MDB_ScheduleItem.Column_UniqueID);
            this.dataAdapter.UpdateCommand.CommandType = CommandType.Text;
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_AllDay, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_AllDay, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Content, OleDbType.LongVarWChar, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Content, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_EndTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_EndTime, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_LabelValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LabelValue, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_LocationValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LocationValue, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_MarkerValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_MarkerValue, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Owner, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Owner, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Reminder, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Reminder, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_ReminderValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_ReminderValue, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_StartTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_StartTime, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Subject, OleDbType.LongVarWChar, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Subject, DataRowVersion.Current, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_UniqueID, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_UniqueID, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_AllDay, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_AllDay, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_AllDay, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_AllDay, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_EndTime, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_EndTime, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_EndTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_EndTime, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_LabelValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LabelValue, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_LabelValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LabelValue, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_LocationValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LocationValue, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_LocationValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LocationValue, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_MarkerValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_MarkerValue, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_MarkerValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_MarkerValue, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_Owner, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Owner, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_Owner, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Owner, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_Reminder, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Reminder, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_Reminder, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Reminder, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_ReminderValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_ReminderValue, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_ReminderValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_ReminderValue, DataRowVersion.Original, false, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_StartTime, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_StartTime, DataRowVersion.Original, true, null));
            this.dataAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_StartTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_StartTime, DataRowVersion.Original, false, null));
            #endregion

            #region Insert Command

            this.dataAdapter.InsertCommand = new OleDbCommand();
            this.dataAdapter.InsertCommand.CommandText = string.Format("INSERT INTO `{0}`(`{1}`, `{2}`, `{3}`, `{4}`, `{5}`, `{6}`, `{7}`, `{8}`, `{9}`, `{10}`, `{11}`) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                                                         Table_Appointments, MDB_ScheduleItem.Column_AllDay, MDB_ScheduleItem.Column_Content, MDB_ScheduleItem.Column_EndTime,
                                                         MDB_ScheduleItem.Column_LabelValue, MDB_ScheduleItem.Column_LocationValue, MDB_ScheduleItem.Column_MarkerValue,
                                                         MDB_ScheduleItem.Column_Owner, MDB_ScheduleItem.Column_Reminder, MDB_ScheduleItem.Column_ReminderValue, MDB_ScheduleItem.Column_StartTime, MDB_ScheduleItem.Column_Subject);
            this.dataAdapter.InsertCommand.CommandType = CommandType.Text;
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_AllDay, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_AllDay, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Content, OleDbType.LongVarWChar, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Content, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_EndTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_EndTime, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_LabelValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LabelValue, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_LocationValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LocationValue, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_MarkerValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_MarkerValue, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Owner, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Owner, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Reminder, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Reminder, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_ReminderValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_ReminderValue, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_StartTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_StartTime, DataRowVersion.Current, false, null));
            this.dataAdapter.InsertCommand.Parameters.Add(new OleDbParameter(MDB_ScheduleItem.Column_Subject, OleDbType.LongVarWChar, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Subject, DataRowVersion.Current, false, null));
           			
            this.dataAdapter.InsertCommand.Connection = this.Connection;

            #endregion

            #region Delete Command

            this.dataAdapter.DeleteCommand = new OleDbCommand();
            this.dataAdapter.DeleteCommand.Connection = this.Connection;
            this.dataAdapter.DeleteCommand.CommandText = @"DELETE FROM `Appointments` WHERE ((`ID` = ?) AND ((? = 1 AND `AllDay` IS NULL) OR (`AllDay` = ?)) AND ((? = 1 AND `EndTime` IS NULL) OR (`EndTime` = ?)) AND ((? = 1 AND `LabelValue` IS NULL) OR (`LabelValue` = ?)) AND ((? = 1 AND `LocationValue` IS NULL) OR (`LocationValue` = ?)) AND ((? = 1 AND `MarkerValue` IS NULL) OR (`MarkerValue` = ?)) AND ((? = 1 AND `Owner` IS NULL) OR (`Owner` = ?)) AND ((? = 1 AND `Reminder` IS NULL) OR (`Reminder` = ?)) AND ((? = 1 AND `ReminderValue` IS NULL) OR (`ReminderValue` = ?)) AND ((? = 1 AND `StartTime` IS NULL) OR (`StartTime` = ?)))";
            this.dataAdapter.DeleteCommand.CommandType = CommandType.Text;
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_UniqueID, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_UniqueID, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_AllDay, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_AllDay, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_AllDay, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_AllDay, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_EndTime, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_EndTime, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_EndTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_EndTime, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_LabelValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LabelValue, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_LabelValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LabelValue, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_LocationValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LocationValue, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_LocationValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_LocationValue, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_MarkerValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_MarkerValue, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_MarkerValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_MarkerValue, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_Owner, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Owner, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_Owner, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Owner, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_Reminder, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Reminder, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_Reminder, OleDbType.Boolean, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_Reminder, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_ReminderValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_ReminderValue, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_ReminderValue, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_ReminderValue, DataRowVersion.Original, false, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("IsNull_" + MDB_ScheduleItem.Column_StartTime, OleDbType.Integer, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_StartTime, DataRowVersion.Original, true, null));
            this.dataAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_" + MDB_ScheduleItem.Column_StartTime, OleDbType.Date, 0, ParameterDirection.Input, ((byte)(0)), ((byte)(0)), MDB_ScheduleItem.Column_StartTime, DataRowVersion.Original, false, null));

            #endregion

            this.dataAdapter.Update(scheduleDataSet, Table_Appointments);
            if (!LoadOnDemand)
            {
                this.connection.Close();
            }
          
        }

        private static LookUpObjectList labels;

        
        internal static LookUpObjectList Labels
        {
            get 
            {
                if (labels == null)
                {
                    labels = new LookUpObjectList(scheduleDataSet.Tables[Table_Labels].DefaultView);
                }
                return MDB_ScheduleDataProvider.labels; 
            }
            set { MDB_ScheduleDataProvider.labels = value; }
        }

        /// <summary>
        /// The Labels lookup list.
        /// </summary>
        /// <returns>The Labels list.</returns>
        public override ILookUpObjectList GetLabels()
        {
            return Labels;
        }
        private static LookUpObjectList locations;

        internal static LookUpObjectList Locations
        {
            get
            {
                if (locations == null)
                {
                    locations = new LookUpObjectList(scheduleDataSet.Tables[Table_Locations].DefaultView);
                }
                return MDB_ScheduleDataProvider.locations;
            }
            set { MDB_ScheduleDataProvider.locations = value; }
        }
        /// <summary>
        /// The Locations lookup list.
        /// </summary>
        /// <returns>The Locations lookup list.</returns>
        public override ILookUpObjectList GetLocations()
        {
            return Locations;
        }
        private static LookUpObjectList markers;

        internal static LookUpObjectList Markers
        {
            get
            {
                if (markers == null)
                {
                    markers = new LookUpObjectList(scheduleDataSet.Tables[Table_Markers].DefaultView);
                }
                return MDB_ScheduleDataProvider.markers;
            }
            set { MDB_ScheduleDataProvider.markers = value; }
        }
        /// <summary>
        /// The Markers lookup list.
        /// </summary>
        /// <returns>The Markers lookup list.</returns>
        public override ILookUpObjectList GetMarkers()
        {
            return Markers;
        }

        //not used in this sample...
        /// <summary>
        /// The owners list.
        /// </summary>
        /// <returns>The owners list.</returns>
        public override IScheduleResourceList GetOwners()
        {
            return base.GetOwners();
        }

        private static LookUpObjectList reminders;

        internal static LookUpObjectList Reminders
        {
            get
            {
                if (reminders == null)
                {
                    reminders = new LookUpObjectList(scheduleDataSet.Tables[Table_Reminders].DefaultView);
                }

                return reminders;
            }
            set { reminders = value; }
        }
        /// <summary>
        /// The Reminders lookup list.
        /// </summary>
        /// <returns>The Reminders lookup list.</returns>
        public override ILookUpObjectList GetReminders()
        {
            return Reminders;
        }
        private string lastWhereLoaded = "";
        /// <summary>
        /// Returns the appointments for a given date range.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="endDate">The end date.</param>
        /// <returns>A list of appointments.</returns>
        /// <remarks>Depending upon the value of LoadOnDemand, this list is either generated
        /// using a RowFilter on the disconnected DataView (when LoadPOnDemand=false) or using
        /// reloading the Appointments DataTable  (when LoadOnDemand=true).
        /// </remarks>
        public override IScheduleAppointmentList GetSchedule(DateTime startDate, DateTime endDate)
        {
            string whereClause = string.Format(System.Globalization.CultureInfo.InvariantCulture, "[{0}] >= #{1}# AND [{2}] < #{3}#", MDB_ScheduleItem.Column_StartTime, startDate.Date,
                   MDB_ScheduleItem.Column_EndTime, endDate.Date.AddDays(1));
            if (!LoadOnDemand)
            {
                DataView dv = new DataView(scheduleDataSet.Tables[Table_Appointments]);
                dv.RowFilter = whereClause;
                return new MDB_ScheduleItemList(dv);
            }
            else
            {
                if (lastWhereLoaded != whereClause)
                {
                    DataTable dt = scheduleDataSet.Tables[Table_Appointments];
                    if (dt != null)
                    {
                        if (dt.GetChanges() != null)
                        {
                            CommitChanges();
                        }
                        dt.Clear();

                    }
                    string sqlString = "SELECT * FROM " + Table_Appointments + " where " + whereClause;
                    dataAdapter = new OleDbDataAdapter(sqlString, Connection);
                    dataAdapter.Fill(scheduleDataSet, Table_Appointments);
                    lastWhereLoaded = whereClause;
                    //raise the event
                    OnDemandLoadExecuted(EventArgs.Empty);
                  
                }

                return new MDB_ScheduleItemList(scheduleDataSet.Tables[Table_Appointments].DefaultView);
            }
        }

        /// <summary>
        /// Event raise only when LoadOnDemand is true and new appointment data has been loaded.
        /// </summary>
        public event EventHandler DemandLoadExecuted;

        /// <summary>
        /// Raises the DemandLoadExecuted event.
        /// </summary>
        /// <param name="e">The EventArgs.</param>
        protected virtual void OnDemandLoadExecuted(EventArgs e)
        {
            if (DemandLoadExecuted != null)
                DemandLoadExecuted(this, e);
        }

        /// <summary>
        /// Overload of GetSchedule.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="endDate">The end date.</param>
        /// <param name="owner">The owner.</param>
        /// <returns>a list of appointments.</returns>
        public override IScheduleAppointmentList GetSchedule(DateTime startDate, DateTime endDate, int owner)
        {
            //owner not supported...
            return GetSchedule(startDate, endDate);
        }
        /// <summary>
        /// Overload of GetSchedule.
        /// </summary>
        /// <param name="day">The day.</param>
        /// <returns>a list of appointments for a day.</returns>
        public override IScheduleAppointmentList GetScheduleForDay(DateTime day)
        {
            return GetSchedule(day, day); ;
        }
        /// <summary>
        /// Overload of GetSchedule.
        /// </summary>
        /// <param name="day">The day.</param>
        /// <param name="owner">The owner.</param>
        /// <returns>a list of appointments for a day.</returns>
        public override IScheduleAppointmentList GetScheduleForDay(DateTime day, int owner)
        {
            //return base.GetScheduleForDay(day, owner);
            //owner not supported
            return GetScheduleForDay(day);
        }
        /// <summary>
        /// Gets whether the Appointments table is dirty using DataTable.GetChanges.
        /// </summary>
        public override bool IsDirty
        {
            get
            {
                DataTable dt = scheduleDataSet.Tables[Table_Appointments].GetChanges();
                return dt != null && dt.Rows.Count > 0;
            }
            set
            {
               // nop
            }
        }
        /// <summary>
        /// Creates a new appointment.
        /// </summary>
        /// <returns></returns>
        public override IScheduleAppointment NewScheduleAppointment()
        {
            DataRow dr = scheduleDataSet.Tables[Table_Appointments].NewRow();

            PopulateDefaults(dr);
            return new MDB_ScheduleItem(dr); ;
        }

        private void PopulateDefaults(DataRow dr)
        {
             dr[MDB_ScheduleItem.Column_AllDay] = false;
             dr[MDB_ScheduleItem.Column_Content] = "";
       // dr[MDB_ScheduleItem.Column_Dirty] = false;
        dr[MDB_ScheduleItem.Column_EndTime] = DateTime.Now;
       // dr[MDB_ScheduleItem.Column_IgnoreChanges] = false;
        dr[MDB_ScheduleItem.Column_LabelValue] = 0;
        dr[MDB_ScheduleItem.Column_LocationValue] = 0;
        dr[MDB_ScheduleItem.Column_MarkerValue] = 0;
        dr[MDB_ScheduleItem.Column_Owner] = 0;
        dr[MDB_ScheduleItem.Column_Reminder] = false;
        dr[MDB_ScheduleItem.Column_ReminderValue] = 0;
        dr[MDB_ScheduleItem.Column_StartTime] = DateTime.Now;
        dr[MDB_ScheduleItem.Column_Subject] = "";
        //dr[MDB_ScheduleItem.Column_UniqueID] = "ID"; 
        }
        

        /// <summary>
        /// Deletes an appointment.
        /// </summary>
        /// <param name="item">The appointment to be deleted.</param>
        public override void RemoveItem(IScheduleAppointment item)
        {
            (item as MDB_ScheduleItem).Dr.Delete();
        }
        /// <summary>
        /// Applies changes in one appointment to another appointment.
        /// </summary>
        /// <param name="appModifiedItem">The appointment with the changes.</param>
        /// <param name="appOriginalItem">The appointment to be changed.</param>
        public override void SaveModifiedItem(IScheduleAppointment appModifiedItem, IScheduleAppointment appOriginalItem)
        {
          
            (appOriginalItem as MDB_ScheduleItem).Dr.ItemArray = (appModifiedItem as MDB_ScheduleItem).Dr.ItemArray.Clone() as object[];
        }
    }

    #endregion

    #region lookup lists

    public class LookUpObjectList : ListObjectList 
    {
        
        public LookUpObjectList(DataView dv)
        {
            foreach (DataRowView drv in dv)
            {
                this.Add(new LookUpObject(drv) as ILookUpObject);
            }
        }
    }

    /// <summary>
    /// Wraps a Lookup member so the color is split into RGB integer values.
    /// </summary>
    public class LookUpObject : ListObject
    {
        /// <summary>
        /// Column name of the ID value for this object.
        /// </summary>
        public static string Column_ID = "ID";
        /// <summary>
        /// The red component of the color for this object.
        /// </summary>
        public static string Column_ColorRed = "ColorRed";
        /// <summary>
        /// The blue component of the color of this object.
        /// </summary>
        public static string Column_ColorBlue = "ColorBlue";
        /// <summary>
        /// The green component of the color of this object.
        /// </summary>
        public static string Column_ColorGreen = "ColorGreen";
        /// <summary>
        /// The display value for this object.
        /// </summary>
        public static string Column_Display = "Display";

        DataRowView drv;
        /// <summary>
        /// Creates a lookup object based on a DataRowView.
        /// </summary>
        /// <param name="drv"></param>
        public LookUpObject(DataRowView drv)
            : base((int)drv[Column_ID], drv[Column_Display].ToString(), Color.FromArgb((int)drv[Column_ColorRed], (int)drv[Column_ColorBlue], (int)drv[Column_ColorGreen]))
        {
            this.drv = drv;
        }

        /// <summary>
        /// Gets or sets the Color associated with this lookup object.
        /// </summary>
        /// <remarks>
        /// The Color is serialized to the underlying DataTable as an RGB integer triple.
        /// </remarks>
        public override Color ColorMember
        {
            get
            {
                return Color.FromArgb((int)drv[Column_ColorRed], (int)drv[Column_ColorBlue], (int)drv[Column_ColorGreen]);
            }
            set
            {
                drv[Column_ColorRed] = value.R;
                drv[Column_ColorBlue] = value.B;
                drv[Column_ColorGreen] = value.G;
            }
        }

        /// <summary>
        /// Gets or sets the DisplayMember for this lookup object.
        /// </summary>
        public override string DisplayMember
        {
            get
            {
                return drv[Column_Display].ToString();
            }
            set
            {
                drv[Column_Display] = value;
            }
        }
        /// <summary>
        /// Gets or sets the ValueMember for this lookup object.
        /// </summary>
        public override int ValueMember
        {
            get
            {
                return (int)drv[Column_ID];
            }
            set
            {
                drv[Column_ID] = value;
            }
        }
    }
    #endregion
}
